@@ -216,10 +216,10 @@ static std::string getTagsData(PHLWINDOW w, eHyprCtlOutputFormat format) {
     const auto tags = w->m_tags.getTags();
 
     if (format == eHyprCtlOutputFormat::FORMAT_JSON)
-        return std::accumulate(tags.begin(), tags.end(), std::string(),
-                               [](const std::string& a, const std::string& b) { return a.empty() ? std::format("\"{}\"", b) : std::format("{}, \"{}\"", a, b); });
+        return std::ranges::fold_left(tags, std::string(),
+                                      [](const std::string& a, const std::string& b) { return a.empty() ? std::format("\"{}\"", b) : std::format("{}, \"{}\"", a, b); });
     else
-        return std::accumulate(tags.begin(), tags.end(), std::string(), [](const std::string& a, const std::string& b) { return a.empty() ? b : a + ", " + b; });
+        return std::ranges::fold_left(tags, std::string(), [](const std::string& a, const std::string& b) { return a.empty() ? b : a + ", " + b; });
 }
 
 static std::string getGroupedData(PHLWINDOW w, eHyprCtlOutputFormat format) {
@@ -331,6 +331,9 @@ static std::string clientsRequest(eHyprCtlOutputFormat format, std::string reque
 
             result += CHyprCtl::getWindowData(w, format);
         }
+
+        if (result.empty())
+            return "no open windows";
     }
     return result;
 }
@@ -516,7 +519,7 @@ static std::string layersRequest(eHyprCtlOutputFormat format, std::string reques
 
                 trimTrailingComma(result);
 
-                if (level.size() > 0)
+                if (!level.empty())
                     result += "\n        ";
 
                 result += "],";
@@ -921,7 +924,7 @@ static std::string bindsRequest(eHyprCtlOutputFormat format, std::string request
 std::string versionRequest(eHyprCtlOutputFormat format, std::string request) {
 
     auto commitMsg = trim(GIT_COMMIT_MESSAGE);
-    std::replace(commitMsg.begin(), commitMsg.end(), '#', ' ');
+    std::ranges::replace(commitMsg, '#', ' ');
 
     if (format == eHyprCtlOutputFormat::FORMAT_NORMAL) {
         std::string result = std::format("Hyprland {} built from branch {} at commit {} {} ({}).\n"
@@ -931,13 +934,10 @@ std::string versionRequest(eHyprCtlOutputFormat format, std::string request) {
                                          HYPRLAND_VERSION, GIT_BRANCH, GIT_COMMIT_HASH, GIT_DIRTY, commitMsg, GIT_COMMIT_DATE, GIT_TAG, GIT_COMMITS, AQUAMARINE_VERSION,
                                          HYPRLANG_VERSION, HYPRUTILS_VERSION, HYPRCURSOR_VERSION, HYPRGRAPHICS_VERSION);
 
-#if (!defined(LEGACY_RENDERER) && !ISDEBUG && !defined(NO_XWAYLAND))
+#if (!ISDEBUG && !defined(NO_XWAYLAND))
         result += "no flags were set\n";
 #else
         result += "flags set:\n";
-#ifdef LEGACY_RENDERER
-        result += "legacyrenderer\n";
-#endif
 #if ISDEBUG
         result += "debug\n";
 #endif
@@ -966,9 +966,6 @@ std::string versionRequest(eHyprCtlOutputFormat format, std::string request) {
             GIT_BRANCH, GIT_COMMIT_HASH, HYPRLAND_VERSION, (strcmp(GIT_DIRTY, "dirty") == 0 ? "true" : "false"), escapeJSONStrings(commitMsg), GIT_COMMIT_DATE, GIT_TAG,
             GIT_COMMITS, AQUAMARINE_VERSION, HYPRLANG_VERSION, HYPRUTILS_VERSION, HYPRCURSOR_VERSION, HYPRGRAPHICS_VERSION);
 
-#ifdef LEGACY_RENDERER
-        result += "\"legacyrenderer\",";
-#endif
 #if ISDEBUG
         result += "\"debug\",";
 #endif
@@ -1115,6 +1112,9 @@ static std::string dispatchKeyword(eHyprCtlOutputFormat format, std::string in)
     if (COMMAND == "monitor" || COMMAND == "source")
         g_pConfigManager->m_wantsMonitorReload = true; // for monitor keywords
 
+    if (COMMAND.contains("monitorv2"))
+        g_pEventLoopManager->doLater([] { g_pConfigManager->m_wantsMonitorReload = true; });
+
     if (COMMAND.contains("input") || COMMAND.contains("device") || COMMAND == "source") {
         g_pInputManager->setKeyboardLayout();     // update kb layout
         g_pInputManager->setPointerConfigs();     // update mouse cfgs
@@ -1142,7 +1142,9 @@ static std::string dispatchKeyword(eHyprCtlOutputFormat format, std::string in)
     // decorations will probably need a repaint
     if (COMMAND.contains("decoration:") || COMMAND.contains("border") || COMMAND == "workspace" || COMMAND.contains("zoom_factor") || COMMAND == "source" ||
         COMMAND.starts_with("windowrule")) {
+        static auto PZOOMFACTOR = CConfigValue<Hyprlang::FLOAT>("cursor:zoom_factor");
         for (auto const& m : g_pCompositor->m_monitors) {
+            *(m->m_cursorZoom) = *PZOOMFACTOR;
             g_pHyprRenderer->damageMonitor(m);
             g_pLayoutManager->getCurrentLayout()->recalculateMonitor(m->m_id);
         }
@@ -1150,7 +1152,7 @@ static std::string dispatchKeyword(eHyprCtlOutputFormat format, std::string in)
 
     Debug::log(LOG, "Hyprctl: keyword {} : {}", COMMAND, VALUE);
 
-    if (retval == "")
+    if (retval.empty())
         return "ok";
 
     return retval;
@@ -1302,8 +1304,7 @@ static std::string switchXKBLayoutRequest(eHyprCtlOutputFormat format, std::stri
         }
         return result.empty() ? "ok" : result;
     } else {
-        auto k = std::find_if(g_pInputManager->m_keyboards.begin(), g_pInputManager->m_keyboards.end(),
-                              [&](const auto& other) { return other->m_hlName == g_pInputManager->deviceNameToInternalString(KB); });
+        auto k = std::ranges::find_if(g_pInputManager->m_keyboards, [&](const auto& other) { return other->m_hlName == g_pInputManager->deviceNameToInternalString(KB); });
 
         if (k == g_pInputManager->m_keyboards.end())
             return "device not found";
@@ -1535,7 +1536,7 @@ static std::string dispatchPlugin(eHyprCtlOutputFormat format, std::string reque
         if (format == eHyprCtlOutputFormat::FORMAT_JSON) {
             result += "[";
 
-            if (PLUGINS.size() == 0)
+            if (PLUGINS.empty())
                 return "[]";
 
             for (auto const& p : PLUGINS) {
@@ -1553,7 +1554,7 @@ static std::string dispatchPlugin(eHyprCtlOutputFormat format, std::string reque
             trimTrailingComma(result);
             result += "]";
         } else {
-            if (PLUGINS.size() == 0)
+            if (PLUGINS.empty())
                 return "no plugins loaded";
 
             for (auto const& p : PLUGINS) {
