@@ -33,7 +33,7 @@ static int wlTick(SP<CEventLoopTimer> self, void* data) {
 }
 
 CHyprAnimationManager::CHyprAnimationManager() {
-    m_animationTimer = SP<CEventLoopTimer>(new CEventLoopTimer(std::chrono::microseconds(500), wlTick, nullptr));
+    m_animationTimer = makeShared<CEventLoopTimer>(std::chrono::microseconds(500), wlTick, nullptr);
     if (g_pEventLoopManager) // null in --verify-config mode
         g_pEventLoopManager->addTimer(m_animationTimer);
 
@@ -306,7 +306,7 @@ void CHyprAnimationManager::animationSlide(PHLWINDOW pWindow, std::string force,
 
     Vector2D posOffset;
 
-    if (force != "") {
+    if (!force.empty()) {
         if (force == "bottom")
             posOffset = Vector2D(GOALPOS.x, PMONITOR->m_position.y + PMONITOR->m_size.y);
         else if (force == "left")
@@ -384,7 +384,7 @@ void CHyprAnimationManager::onWindowPostCreateClose(PHLWINDOW pWindow, bool clos
     }
 
     std::string ANIMSTYLE = pWindow->m_realPosition->getStyle();
-    transform(ANIMSTYLE.begin(), ANIMSTYLE.end(), ANIMSTYLE.begin(), ::tolower);
+    std::ranges::transform(ANIMSTYLE, ANIMSTYLE.begin(), ::tolower);
 
     CVarList animList(ANIMSTYLE, 0, 's');
 
@@ -490,7 +490,7 @@ std::string CHyprAnimationManager::styleValidInConfigVar(const std::string& conf
             return "";
         return "unknown style";
     } else if (config.starts_with("layers")) {
-        if (style == "fade" || style == "" || style == "slide")
+        if (style.empty() || style == "fade" || style == "slide")
             return "";
         else if (style.starts_with("popin")) {
             // try parsing
