@@ -3,16 +3,17 @@
 #include "../config/ConfigValue.hpp"
 #include "../protocols/FractionalScale.hpp"
 #include "../protocols/SessionLock.hpp"
-#include "../managers/SeatManager.hpp"
 #include "../render/Renderer.hpp"
-#include "../managers/input/InputManager.hpp"
+#include "./managers/SeatManager.hpp"
+#include "./managers/input/InputManager.hpp"
+#include "./managers/eventLoop/EventLoopManager.hpp"
 #include <algorithm>
 #include <ranges>
 
 SSessionLockSurface::SSessionLockSurface(SP<CSessionLockSurface> surface_) : surface(surface_) {
     pWlrSurface = surface->surface();
 
-    listeners.map = surface_->m_events.map.registerListener([this](std::any data) {
+    listeners.map = surface_->m_events.map.listen([this] {
         mapped = true;
 
         g_pInputManager->simulateMouseMovement();
@@ -23,14 +24,14 @@ SSessionLockSurface::SSessionLockSurface(SP<CSessionLockSurface> surface_) : sur
             g_pHyprRenderer->damageMonitor(PMONITOR);
     });
 
-    listeners.destroy = surface_->m_events.destroy.registerListener([this](std::any data) {
+    listeners.destroy = surface_->m_events.destroy.listen([this] {
         if (pWlrSurface == g_pCompositor->m_lastFocus)
             g_pCompositor->m_lastFocus.reset();
 
         g_pSessionLockManager->removeSessionLockSurface(this);
     });
 
-    listeners.commit = surface_->m_events.commit.registerListener([this](std::any data) {
+    listeners.commit = surface_->m_events.commit.listen([this] {
         const auto PMONITOR = g_pCompositor->getMonitorFromID(iMonitorID);
 
         if (mapped && !g_pCompositor->m_lastFocus)
@@ -42,36 +43,36 @@ SSessionLockSurface::SSessionLockSurface(SP<CSessionLockSurface> surface_) : sur
 }
 
 CSessionLockManager::CSessionLockManager() {
-    m_listeners.newLock = PROTO::sessionLock->m_events.newLock.registerListener([this](std::any data) { this->onNewSessionLock(std::any_cast<SP<CSessionLock>>(data)); });
+    m_listeners.newLock = PROTO::sessionLock->m_events.newLock.listen([this](const auto& lock) { this->onNewSessionLock(lock); });
 }
 
 void CSessionLockManager::onNewSessionLock(SP<CSessionLock> pLock) {
-
     static auto PALLOWRELOCK = CConfigValue<Hyprlang::INT>("misc:allow_session_lock_restore");
 
     if (PROTO::sessionLock->isLocked() && !*PALLOWRELOCK) {
-        Debug::log(LOG, "Cannot re-lock, misc:allow_session_lock_restore is disabled");
+        LOGM(LOG, "Cannot re-lock, misc:allow_session_lock_restore is disabled");
         pLock->sendDenied();
         return;
     }
 
-    Debug::log(LOG, "Session got locked by {:x}", (uintptr_t)pLock.get());
+    if (m_sessionLock && !clientDenied() && !clientLocked())
+        return; // Not allowing to relock in case the old lock is still in a limbo
+
+    LOGM(LOG, "Session got locked by {:x}", (uintptr_t)pLock.get());
 
     m_sessionLock       = makeUnique<SSessionLock>();
     m_sessionLock->lock = pLock;
-    m_sessionLock->mLockTimer.reset();
+    m_sessionLock->lockTimer.reset();
 
-    m_sessionLock->listeners.newSurface = pLock->m_events.newLockSurface.registerListener([this](std::any data) {
-        auto       SURFACE = std::any_cast<SP<CSessionLockSurface>>(data);
+    m_sessionLock->listeners.newSurface = pLock->m_events.newLockSurface.listen([this](const SP<CSessionLockSurface>& surface) {
+        const auto PMONITOR = surface->monitor();
 
-        const auto PMONITOR = SURFACE->monitor();
-
-        const auto NEWSURFACE  = m_sessionLock->vSessionLockSurfaces.emplace_back(makeUnique<SSessionLockSurface>(SURFACE)).get();
+        const auto NEWSURFACE  = m_sessionLock->vSessionLockSurfaces.emplace_back(makeUnique<SSessionLockSurface>(surface)).get();
         NEWSURFACE->iMonitorID = PMONITOR->m_id;
-        PROTO::fractional->sendScale(SURFACE->surface(), PMONITOR->m_scale);
+        PROTO::fractional->sendScale(surface->surface(), PMONITOR->m_scale);
     });
 
-    m_sessionLock->listeners.unlock = pLock->m_events.unlockAndDestroy.registerListener([this](std::any data) {
+    m_sessionLock->listeners.unlock = pLock->m_events.unlockAndDestroy.listen([this] {
         m_sessionLock.reset();
         g_pInputManager->refocus();
 
@@ -79,7 +80,7 @@ void CSessionLockManager::onNewSessionLock(SP<CSessionLock> pLock) {
             g_pHyprRenderer->damageMonitor(m);
     });
 
-    m_sessionLock->listeners.destroy = pLock->m_events.destroyed.registerListener([this](std::any data) {
+    m_sessionLock->listeners.destroy = pLock->m_events.destroyed.listen([this] {
         m_sessionLock.reset();
         g_pCompositor->focusSurface(nullptr);
 
@@ -90,14 +91,46 @@ void CSessionLockManager::onNewSessionLock(SP<CSessionLock> pLock) {
     g_pCompositor->focusSurface(nullptr);
     g_pSeatManager->setGrab(nullptr);
 
-    // Normally the locked event is sent after each output rendered a lock screen frame.
-    // When there are no outputs, send it right away.
     if (g_pCompositor->m_unsafeState) {
+        m_sessionLock->sendDeniedTimer = makeShared<CEventLoopTimer>(
+            // Within this arbitrary amount of time, a session-lock client is expected to create and commit a lock surface for each output. If the client fails to do that, it will be denied.
+            std::chrono::seconds(5),
+            [](auto, auto) {
+                if (!g_pSessionLockManager || g_pSessionLockManager->clientLocked() || g_pSessionLockManager->clientDenied())
+                    return;
+
+                if (g_pCompositor->m_unsafeState || !g_pCompositor->m_aqBackend->hasSession() || !g_pCompositor->m_aqBackend->session->active) {
+                    // Because the session is inactive, there is a good reason for why the client did't recieve locked or denied.
+                    // We send locked, although this could lead to imperfect frames when we start to render again.
+                    g_pSessionLockManager->m_sessionLock->lock->sendLocked();
+                    g_pSessionLockManager->m_sessionLock->hasSentLocked = true;
+                    return;
+                }
+
+                if (g_pSessionLockManager->m_sessionLock && g_pSessionLockManager->m_sessionLock->lock) {
+                    g_pSessionLockManager->m_sessionLock->lock->sendDenied();
+                    g_pSessionLockManager->m_sessionLock->hasSentDenied = true;
+                }
+            },
+            nullptr);
+
+        g_pEventLoopManager->addTimer(m_sessionLock->sendDeniedTimer);
+    } else {
+        // Normally the locked event is sent after each output rendered a lock screen frame.
+        // When there are no outputs, send it right away.
         m_sessionLock->lock->sendLocked();
         m_sessionLock->hasSentLocked = true;
     }
 }
 
+void CSessionLockManager::removeSendDeniedTimer() {
+    if (!m_sessionLock || !m_sessionLock->sendDeniedTimer)
+        return;
+
+    g_pEventLoopManager->removeTimer(m_sessionLock->sendDeniedTimer);
+    m_sessionLock->sendDeniedTimer.reset();
+}
+
 bool CSessionLockManager::isSessionLocked() {
     return PROTO::sessionLock->isLocked();
 }
@@ -118,28 +151,13 @@ WP<SSessionLockSurface> CSessionLockManager::getSessionLockSurfaceForMonitor(uin
     return {};
 }
 
-// We don't want the red screen to flash.
-float CSessionLockManager::getRedScreenAlphaForMonitor(uint64_t id) {
-    if (!m_sessionLock)
-        return 1.F;
-
-    const auto& NOMAPPEDSURFACETIMER = m_sessionLock->mMonitorsWithoutMappedSurfaceTimers.find(id);
-
-    if (NOMAPPEDSURFACETIMER == m_sessionLock->mMonitorsWithoutMappedSurfaceTimers.end()) {
-        m_sessionLock->mMonitorsWithoutMappedSurfaceTimers.emplace(id, CTimer());
-        m_sessionLock->mMonitorsWithoutMappedSurfaceTimers[id].reset();
-        return 0.f;
-    }
-
-    return std::clamp(NOMAPPEDSURFACETIMER->second.getSeconds() - /* delay for screencopy */ 0.5f, 0.f, 1.f);
-}
-
 void CSessionLockManager::onLockscreenRenderedOnMonitor(uint64_t id) {
-    if (!m_sessionLock || m_sessionLock->hasSentLocked)
+    if (!m_sessionLock || m_sessionLock->hasSentLocked || m_sessionLock->hasSentDenied)
         return;
     m_sessionLock->lockedMonitors.emplace(id);
     const bool LOCKED = std::ranges::all_of(g_pCompositor->m_monitors, [this](auto m) { return m_sessionLock->lockedMonitors.contains(m->m_id); });
     if (LOCKED && m_sessionLock->lock->good()) {
+        removeSendDeniedTimer();
         m_sessionLock->lock->sendLocked();
         m_sessionLock->hasSentLocked = true;
     }
@@ -160,6 +178,10 @@ bool CSessionLockManager::isSurfaceSessionLock(SP<CWLSurfaceResource> pSurface)
     return false;
 }
 
+bool CSessionLockManager::anySessionLockSurfacesPresent() {
+    return m_sessionLock && std::ranges::any_of(m_sessionLock->vSessionLockSurfaces, [](const auto& surf) { return surf->mapped; });
+}
+
 void CSessionLockManager::removeSessionLockSurface(SSessionLockSurface* pSLS) {
     if (!m_sessionLock)
         return;
@@ -178,19 +200,19 @@ void CSessionLockManager::removeSessionLockSurface(SSessionLockSurface* pSLS) {
     }
 }
 
-bool CSessionLockManager::isSessionLockPresent() {
-    return m_sessionLock && !m_sessionLock->vSessionLockSurfaces.empty();
+bool CSessionLockManager::clientLocked() {
+    return m_sessionLock && m_sessionLock->hasSentLocked;
 }
 
-bool CSessionLockManager::anySessionLockSurfacesPresent() {
-    return m_sessionLock && std::ranges::any_of(m_sessionLock->vSessionLockSurfaces, [](const auto& surf) { return surf->mapped; });
+bool CSessionLockManager::clientDenied() {
+    return m_sessionLock && m_sessionLock->hasSentDenied;
 }
 
 bool CSessionLockManager::shallConsiderLockMissing() {
     if (!m_sessionLock)
-        return false;
+        return true;
 
     static auto LOCKDEAD_SCREEN_DELAY = CConfigValue<Hyprlang::INT>("misc:lockdead_screen_delay");
 
-    return m_sessionLock->mLockTimer.getMillis() > *LOCKDEAD_SCREEN_DELAY;
+    return m_sessionLock->lockTimer.getMillis() > *LOCKDEAD_SCREEN_DELAY;
 }
