@@ -281,8 +281,8 @@ CColorManagementSurface::CColorManagementSurface(SP<CWpColorManagementSurfaceV1>
             return;
         }
 
-        const auto imageDescription = std::find_if(PROTO::colorManagement->m_imageDescriptions.begin(), PROTO::colorManagement->m_imageDescriptions.end(),
-                                                   [&](const auto& other) { return other->resource()->resource() == image_description; });
+        const auto imageDescription =
+            std::ranges::find_if(PROTO::colorManagement->m_imageDescriptions, [&](const auto& other) { return other->resource()->resource() == image_description; });
         if (imageDescription == PROTO::colorManagement->m_imageDescriptions.end()) {
             r->error(WP_COLOR_MANAGEMENT_SURFACE_V1_ERROR_IMAGE_DESCRIPTION, "Image description not found");
             return;
@@ -362,6 +362,11 @@ CColorManagementFeedbackSurface::CColorManagementFeedbackSurface(SP<CWpColorMana
     m_resource->setGetPreferred([this](CWpColorManagementSurfaceFeedbackV1* r, uint32_t id) {
         LOGM(TRACE, "Get preferred for id {}", id);
 
+        if (m_surface.expired()) {
+            r->error(WP_COLOR_MANAGEMENT_SURFACE_FEEDBACK_V1_ERROR_INERT, "Surface is inert");
+            return;
+        }
+
         if (m_currentPreferred.valid())
             PROTO::colorManagement->destroyResource(m_currentPreferred.get());
 
@@ -377,15 +382,15 @@ CColorManagementFeedbackSurface::CColorManagementFeedbackSurface(SP<CWpColorMana
         RESOURCE->m_self   = RESOURCE;
         m_currentPreferred = RESOURCE;
 
-        m_currentPreferred->m_settings = g_pCompositor->getPreferredImageDescription();
+        m_currentPreferred->m_settings = m_surface->getPreferredImageDescription();
         RESOURCE->resource()->sendReady(m_currentPreferred->m_settings.updateId());
     });
 
     m_resource->setGetPreferredParametric([this](CWpColorManagementSurfaceFeedbackV1* r, uint32_t id) {
         LOGM(TRACE, "Get preferred for id {}", id);
 
-        if (!PROTO::colorManagement->m_debug) {
-            r->error(WP_COLOR_MANAGER_V1_ERROR_UNSUPPORTED_FEATURE, "Parametric descriptions are not supported");
+        if (m_surface.expired()) {
+            r->error(WP_COLOR_MANAGEMENT_SURFACE_FEEDBACK_V1_ERROR_INERT, "Surface is inert");
             return;
         }
 
@@ -404,15 +409,28 @@ CColorManagementFeedbackSurface::CColorManagementFeedbackSurface(SP<CWpColorMana
         RESOURCE->m_self   = RESOURCE;
         m_currentPreferred = RESOURCE;
 
-        m_currentPreferred->m_settings = g_pCompositor->getPreferredImageDescription();
-        if (!PROTO::colorManagement->m_debug && m_currentPreferred->m_settings.icc.fd) {
+        m_currentPreferred->m_settings = m_surface->getPreferredImageDescription();
+        m_currentPreferredId           = m_currentPreferred->m_settings.updateId();
+
+        if (!PROTO::colorManagement->m_debug && m_currentPreferred->m_settings.icc.fd >= 0) {
             LOGM(ERR, "FIXME: parse icc profile");
             r->error(WP_COLOR_MANAGER_V1_ERROR_UNSUPPORTED_FEATURE, "ICC profiles are not supported");
             return;
         }
 
-        RESOURCE->resource()->sendReady(m_currentPreferred->m_settings.updateId());
+        RESOURCE->resource()->sendReady(m_currentPreferredId);
     });
+
+    m_listeners.enter = m_surface->m_events.enter.listen([this](const auto& monitor) { onPreferredChanged(); });
+    m_listeners.leave = m_surface->m_events.leave.listen([this](const auto& monitor) { onPreferredChanged(); });
+}
+
+void CColorManagementFeedbackSurface::onPreferredChanged() {
+    if (m_surface->m_enteredOutputs.size() == 1) {
+        const auto newId = m_surface->getPreferredImageDescription().updateId();
+        if (m_currentPreferredId != newId)
+            m_resource->sendPreferredChanged(newId);
+    }
 }
 
 bool CColorManagementFeedbackSurface::good() {
@@ -608,7 +626,7 @@ CColorManagementParametricCreator::CColorManagementParametricCreator(SP<CWpImage
             r->error(WP_IMAGE_DESCRIPTION_CREATOR_PARAMS_V1_ERROR_ALREADY_SET, "Luminances already set");
             return;
         }
-        if (max_lum < reference_lum || reference_lum <= min) {
+        if (max_lum <= min || reference_lum <= min) {
             r->error(WP_IMAGE_DESCRIPTION_CREATOR_PARAMS_V1_ERROR_INVALID_LUMINANCE, "Invalid luminances");
             return;
         }
@@ -791,6 +809,9 @@ void CColorManagementProtocol::onMonitorImageDescriptionChanged(WP<CMonitor> mon
         if (output->m_monitor == monitor)
             output->m_resource->sendImageDescriptionChanged();
     }
+    // recheck feedbacks
+    for (auto const& feedback : m_feedbackSurfaces)
+        feedback->onPreferredChanged();
 }
 
 void CColorManagementProtocol::destroyResource(CColorManager* resource) {
