@@ -13,6 +13,7 @@
 #include "types/Buffer.hpp"
 #include "../helpers/Format.hpp"
 #include "../helpers/time/Time.hpp"
+#include "XDGShell.hpp"
 
 #include <algorithm>
 #include <functional>
@@ -62,11 +63,13 @@ CScreencopyFrame::CScreencopyFrame(SP<CZwlrScreencopyFrameV1> resource_, int32_t
 
     if (box_.width == 0 && box_.height == 0)
         m_box = {0, 0, (int)(m_monitor->m_size.x), (int)(m_monitor->m_size.y)};
-    else {
+    else
         m_box = box_;
-    }
 
-    m_box.transform(wlTransformToHyprutils(m_monitor->m_transform), m_monitor->m_transformedSize.x, m_monitor->m_transformedSize.y).scale(m_monitor->m_scale).round();
+    const auto POS = m_box.pos();
+    m_box.transform(wlTransformToHyprutils(m_monitor->m_transform), m_monitor->m_pixelSize.x, m_monitor->m_pixelSize.y).scale(m_monitor->m_scale).round();
+    m_box.x = POS.x;
+    m_box.y = POS.y;
 
     m_shmStride = NFormatUtils::minStride(PSHMINFO, m_box.w);
 
@@ -187,9 +190,96 @@ void CScreencopyFrame::share() {
         callback(copyShm());
 }
 
+void CScreencopyFrame::renderMon() {
+    auto    TEXTURE = makeShared<CTexture>(m_monitor->m_output->state->state().buffer);
+
+    CRegion fakeDamage = {0, 0, INT16_MAX, INT16_MAX};
+
+    CBox    monbox = CBox{0, 0, m_monitor->m_pixelSize.x, m_monitor->m_pixelSize.y}
+                      .translate({-m_box.x, -m_box.y}) // vvvv kinda ass-backwards but that's how I designed the renderer... sigh.
+                      .transform(wlTransformToHyprutils(invertTransform(m_monitor->m_transform)), m_monitor->m_pixelSize.x, m_monitor->m_pixelSize.y);
+    g_pHyprOpenGL->pushMonitorTransformEnabled(true);
+    g_pHyprOpenGL->setRenderModifEnabled(false);
+    g_pHyprOpenGL->renderTexture(TEXTURE, monbox, 1);
+    g_pHyprOpenGL->setRenderModifEnabled(true);
+    g_pHyprOpenGL->popMonitorTransformEnabled();
+
+    for (auto const& w : g_pCompositor->m_windows) {
+        if (!w->m_windowData.noScreenShare.valueOrDefault())
+            continue;
+
+        if (!g_pHyprRenderer->shouldRenderWindow(w, m_monitor.lock()))
+            continue;
+
+        if (w->isHidden())
+            continue;
+
+        const auto PWORKSPACE = w->m_workspace;
+
+        if UNLIKELY (!PWORKSPACE)
+            continue;
+
+        const auto REALPOS          = w->m_realPosition->value() + (w->m_pinned ? Vector2D{} : PWORKSPACE->m_renderOffset->value());
+        const auto noScreenShareBox = CBox{REALPOS.x, REALPOS.y, std::max(w->m_realSize->value().x, 5.0), std::max(w->m_realSize->value().y, 5.0)}
+                                          .translate(-m_monitor->m_position)
+                                          .scale(m_monitor->m_scale)
+                                          .translate(-m_box.pos());
+
+        const auto dontRound     = w->isEffectiveInternalFSMode(FSMODE_FULLSCREEN) || w->m_windowData.noRounding.valueOrDefault();
+        const auto rounding      = dontRound ? 0 : w->rounding() * m_monitor->m_scale;
+        const auto roundingPower = dontRound ? 2.0f : w->roundingPower();
+
+        g_pHyprOpenGL->renderRect(noScreenShareBox, Colors::BLACK, rounding, roundingPower);
+
+        if (w->m_isX11 || !w->m_popupHead)
+            continue;
+
+        const auto     geom            = w->m_xdgSurface->m_current.geometry;
+        const Vector2D popupBaseOffset = REALPOS - Vector2D{geom.pos().x, geom.pos().y};
+
+        w->m_popupHead->breadthfirst(
+            [&](WP<CPopup> popup, void*) {
+                if (!popup->m_wlSurface || !popup->m_wlSurface->resource() || !popup->m_mapped)
+                    return;
+
+                const auto popRel = popup->coordsRelativeToParent();
+                popup->m_wlSurface->resource()->breadthfirst(
+                    [&](SP<CWLSurfaceResource> surf, const Vector2D& localOff, void*) {
+                        const auto size    = surf->m_current.size;
+                        const auto surfBox = CBox{popupBaseOffset + popRel + localOff, size}.translate(-m_monitor->m_position).scale(m_monitor->m_scale).translate(-m_box.pos());
+
+                        if LIKELY (surfBox.w > 0 && surfBox.h > 0)
+                            g_pHyprOpenGL->renderRect(surfBox, Colors::BLACK);
+                    },
+                    nullptr);
+            },
+            nullptr);
+    }
+
+    if (m_overlayCursor)
+        g_pPointerManager->renderSoftwareCursorsFor(m_monitor.lock(), Time::steadyNow(), fakeDamage,
+                                                    g_pInputManager->getMouseCoordsInternal() - m_monitor->m_position - m_box.pos() / m_monitor->m_scale, true);
+}
+
+void CScreencopyFrame::storeTempFB() {
+    g_pHyprRenderer->makeEGLCurrent();
+
+    m_tempFb.alloc(m_box.w, m_box.h);
+
+    CRegion fakeDamage = {0, 0, INT16_MAX, INT16_MAX};
+
+    if (!g_pHyprRenderer->beginRender(m_monitor.lock(), fakeDamage, RENDER_MODE_FULL_FAKE, nullptr, &m_tempFb, true)) {
+        LOGM(ERR, "Can't copy: failed to begin rendering to temp fb");
+        return;
+    }
+
+    renderMon();
+
+    g_pHyprRenderer->endRender();
+}
+
 void CScreencopyFrame::copyDmabuf(std::function<void(bool)> callback) {
-    const auto PERM    = g_pDynamicPermissionManager->clientPermissionMode(m_resource->client(), PERMISSION_TYPE_SCREENCOPY);
-    auto       TEXTURE = makeShared<CTexture>(m_monitor->m_output->state->state().buffer);
+    const auto PERM = g_pDynamicPermissionManager->clientPermissionMode(m_resource->client(), PERMISSION_TYPE_SCREENCOPY);
 
     CRegion    fakeDamage = {0, 0, INT16_MAX, INT16_MAX};
 
@@ -200,17 +290,12 @@ void CScreencopyFrame::copyDmabuf(std::function<void(bool)> callback) {
     }
 
     if (PERM == PERMISSION_RULE_ALLOW_MODE_ALLOW) {
-        CBox monbox = CBox{0, 0, m_monitor->m_pixelSize.x, m_monitor->m_pixelSize.y}
-                          .translate({-m_box.x, -m_box.y}) // vvvv kinda ass-backwards but that's how I designed the renderer... sigh.
-                          .transform(wlTransformToHyprutils(invertTransform(m_monitor->m_transform)), m_monitor->m_pixelSize.x, m_monitor->m_pixelSize.y);
-        g_pHyprOpenGL->setMonitorTransformEnabled(true);
-        g_pHyprOpenGL->setRenderModifEnabled(false);
-        g_pHyprOpenGL->renderTexture(TEXTURE, monbox, 1);
-        g_pHyprOpenGL->setRenderModifEnabled(true);
-        g_pHyprOpenGL->setMonitorTransformEnabled(false);
-        if (m_overlayCursor)
-            g_pPointerManager->renderSoftwareCursorsFor(m_monitor.lock(), Time::steadyNow(), fakeDamage,
-                                                        g_pInputManager->getMouseCoordsInternal() - m_monitor->m_position - m_box.pos(), true);
+        if (m_tempFb.isAllocated()) {
+            CBox texbox = {{}, m_box.size()};
+            g_pHyprOpenGL->renderTexture(m_tempFb.getTexture(), texbox, 1);
+            m_tempFb.release();
+        } else
+            renderMon();
     } else if (PERM == PERMISSION_RULE_ALLOW_MODE_PENDING)
         g_pHyprOpenGL->clear(Colors::BLACK);
     else {
@@ -228,8 +313,7 @@ void CScreencopyFrame::copyDmabuf(std::function<void(bool)> callback) {
 }
 
 bool CScreencopyFrame::copyShm() {
-    const auto PERM    = g_pDynamicPermissionManager->clientPermissionMode(m_resource->client(), PERMISSION_TYPE_SCREENCOPY);
-    auto       TEXTURE = makeShared<CTexture>(m_monitor->m_output->state->state().buffer);
+    const auto PERM = g_pDynamicPermissionManager->clientPermissionMode(m_resource->client(), PERMISSION_TYPE_SCREENCOPY);
 
     auto       shm                = m_buffer->shm();
     auto [pixelData, fmt, bufLen] = m_buffer->beginDataPtr(0); // no need for end, cuz it's shm
@@ -247,15 +331,12 @@ bool CScreencopyFrame::copyShm() {
     }
 
     if (PERM == PERMISSION_RULE_ALLOW_MODE_ALLOW) {
-        CBox monbox = CBox{0, 0, m_monitor->m_transformedSize.x, m_monitor->m_transformedSize.y}.translate({-m_box.x, -m_box.y});
-        g_pHyprOpenGL->setMonitorTransformEnabled(true);
-        g_pHyprOpenGL->setRenderModifEnabled(false);
-        g_pHyprOpenGL->renderTexture(TEXTURE, monbox, 1);
-        g_pHyprOpenGL->setRenderModifEnabled(true);
-        g_pHyprOpenGL->setMonitorTransformEnabled(false);
-        if (m_overlayCursor)
-            g_pPointerManager->renderSoftwareCursorsFor(m_monitor.lock(), Time::steadyNow(), fakeDamage,
-                                                        g_pInputManager->getMouseCoordsInternal() - m_monitor->m_position - m_box.pos(), true);
+        if (m_tempFb.isAllocated()) {
+            CBox texbox = {{}, m_box.size()};
+            g_pHyprOpenGL->renderTexture(m_tempFb.getTexture(), texbox, 1);
+            m_tempFb.release();
+        } else
+            renderMon();
     } else if (PERM == PERMISSION_RULE_ALLOW_MODE_PENDING)
         g_pHyprOpenGL->clear(Colors::BLACK);
     else {
@@ -264,11 +345,7 @@ bool CScreencopyFrame::copyShm() {
         g_pHyprOpenGL->renderTexture(g_pHyprOpenGL->m_screencopyDeniedTexture, texbox, 1);
     }
 
-#ifndef GLES2
     glBindFramebuffer(GL_READ_FRAMEBUFFER, fb.getFBID());
-#else
-    glBindFramebuffer(GL_FRAMEBUFFER, fb.getFBID());
-#endif
 
     const auto PFORMAT = NFormatUtils::getPixelFormatFromDRM(shm.format);
     if (!PFORMAT) {
@@ -304,11 +381,7 @@ bool CScreencopyFrame::copyShm() {
 
     g_pHyprOpenGL->m_renderData.pMonitor.reset();
 
-#ifndef GLES2
     glBindFramebuffer(GL_READ_FRAMEBUFFER, 0);
-#else
-    glBindFramebuffer(GL_FRAMEBUFFER, 0);
-#endif
 
     LOGM(TRACE, "Copied frame via shm");
 
@@ -428,8 +501,12 @@ void CScreencopyProtocol::onOutputCommit(PHLMONITOR pMonitor) {
         // check permissions
         const auto PERM = g_pDynamicPermissionManager->clientPermissionMode(f->m_resource->client(), PERMISSION_TYPE_SCREENCOPY);
 
-        if (PERM == PERMISSION_RULE_ALLOW_MODE_PENDING)
+        if (PERM == PERMISSION_RULE_ALLOW_MODE_PENDING) {
+            if (!f->m_tempFb.isAllocated())
+                f->storeTempFB(); // make a snapshot before the popup
+
             continue; // pending an answer, don't do anything yet.
+        }
 
         // otherwise share. If it's denied, it will be black.
 
