@@ -1,4 +1,5 @@
 #include "Compositor.hpp"
+#include "../../Compositor.hpp"
 #include "Output.hpp"
 #include "Seat.hpp"
 #include "../types/WLBuffer.hpp"
@@ -17,6 +18,8 @@
 #include "render/Texture.hpp"
 #include <cstring>
 
+using namespace NColorManagement;
+
 class CDefaultSurfaceRole : public ISurfaceRole {
   public:
     virtual eSurfaceRole role() {
@@ -72,8 +75,8 @@ CWLSurfaceResource::CWLSurfaceResource(SP<CWlSurface> resource_) : m_resource(re
     m_resource->setOnDestroy([this](CWlSurface* r) { destroy(); });
 
     m_resource->setAttach([this](CWlSurface* r, wl_resource* buffer, int32_t x, int32_t y) {
-        m_pending.updated.buffer = true;
-        m_pending.updated.offset = true;
+        m_pending.updated.bits.buffer = true;
+        m_pending.updated.bits.offset = true;
 
         m_pending.offset = {x, y};
 
@@ -95,8 +98,8 @@ CWLSurfaceResource::CWLSurfaceResource(SP<CWlSurface> resource_) : m_resource(re
         }
 
         if (m_pending.bufferSize != m_current.bufferSize) {
-            m_pending.updated.damage = true;
-            m_pending.bufferDamage   = CBox{{}, {INT32_MAX, INT32_MAX}};
+            m_pending.updated.bits.damage = true;
+            m_pending.bufferDamage        = CBox{{}, m_pending.bufferSize};
         }
     });
 
@@ -124,10 +127,17 @@ CWLSurfaceResource::CWLSurfaceResource(SP<CWlSurface> resource_) : m_resource(re
             return;
         }
 
-        if ((!m_pending.updated.buffer) ||            // no new buffer attached
+        if ((!m_pending.updated.bits.buffer) ||       // no new buffer attached
             (!m_pending.buffer && !m_pending.texture) // null buffer attached
         ) {
             commitState(m_pending);
+
+            if (!m_pending.buffer && !m_pending.texture) {
+                // null buffer attached, remove any pending states.
+                while (!m_pendingStates.empty()) {
+                    m_pendingStates.pop();
+                }
+            }
             m_pending.reset();
             return;
         }
@@ -149,7 +159,7 @@ CWLSurfaceResource::CWLSurfaceResource(SP<CWlSurface> resource_) : m_resource(re
             m_pendingStates.pop();
         };
 
-        if (state->updated.acquire) {
+        if (state->updated.bits.acquire) {
             // wait on acquire point for this surface, from explicit sync protocol
             state->acquire.addWaiter(whenReadable);
         } else if (state->buffer->isSynchronous()) {
@@ -170,41 +180,46 @@ CWLSurfaceResource::CWLSurfaceResource(SP<CWlSurface> resource_) : m_resource(re
     });
 
     m_resource->setDamage([this](CWlSurface* r, int32_t x, int32_t y, int32_t w, int32_t h) {
-        m_pending.updated.damage = true;
+        m_pending.updated.bits.damage = true;
         m_pending.damage.add(CBox{x, y, w, h});
     });
     m_resource->setDamageBuffer([this](CWlSurface* r, int32_t x, int32_t y, int32_t w, int32_t h) {
-        m_pending.updated.damage = true;
-        m_pending.bufferDamage.add(CBox{x, y, w, h});
+        m_pending.updated.bits.damage = true;
+        const auto damageSize         = Vector2D(w, h);
+
+        if (damageSize > m_pending.bufferSize)
+            m_pending.bufferDamage.add(CBox{{x, y}, m_pending.bufferSize});
+        else
+            m_pending.bufferDamage.add(CBox{{x, y}, damageSize});
     });
 
     m_resource->setSetBufferScale([this](CWlSurface* r, int32_t scale) {
         if (scale == m_pending.scale)
             return;
 
-        m_pending.updated.scale  = true;
-        m_pending.updated.damage = true;
+        m_pending.updated.bits.scale  = true;
+        m_pending.updated.bits.damage = true;
 
         m_pending.scale        = scale;
-        m_pending.bufferDamage = CBox{{}, {INT32_MAX, INT32_MAX}};
+        m_pending.bufferDamage = CBox{{}, m_pending.bufferSize};
     });
 
     m_resource->setSetBufferTransform([this](CWlSurface* r, uint32_t tr) {
         if (tr == m_pending.transform)
             return;
 
-        m_pending.updated.transform = true;
-        m_pending.updated.damage    = true;
+        m_pending.updated.bits.transform = true;
+        m_pending.updated.bits.damage    = true;
 
         m_pending.transform    = (wl_output_transform)tr;
-        m_pending.bufferDamage = CBox{{}, {INT32_MAX, INT32_MAX}};
+        m_pending.bufferDamage = CBox{{}, m_pending.bufferSize};
     });
 
     m_resource->setSetInputRegion([this](CWlSurface* r, wl_resource* region) {
-        m_pending.updated.input = true;
+        m_pending.updated.bits.input = true;
 
         if (!region) {
-            m_pending.input = CBox{{}, {INT32_MAX, INT32_MAX}};
+            m_pending.input = CBox{{}, Vector2D{INT32_MAX - 1, INT32_MAX - 1}};
             return;
         }
 
@@ -213,7 +228,7 @@ CWLSurfaceResource::CWLSurfaceResource(SP<CWlSurface> resource_) : m_resource(re
     });
 
     m_resource->setSetOpaqueRegion([this](CWlSurface* r, wl_resource* region) {
-        m_pending.updated.opaque = true;
+        m_pending.updated.bits.opaque = true;
 
         if (!region) {
             m_pending.opaque = CBox{{}, {}};
@@ -227,8 +242,8 @@ CWLSurfaceResource::CWLSurfaceResource(SP<CWlSurface> resource_) : m_resource(re
     m_resource->setFrame([this](CWlSurface* r, uint32_t id) { m_callbacks.emplace_back(makeShared<CWLCallbackResource>(makeShared<CWlCallback>(m_client, 1, id))); });
 
     m_resource->setOffset([this](CWlSurface* r, int32_t x, int32_t y) {
-        m_pending.updated.offset = true;
-        m_pending.offset         = {x, y};
+        m_pending.updated.bits.offset = true;
+        m_pending.offset              = {x, y};
     });
 }
 
@@ -268,7 +283,7 @@ wl_client* CWLSurfaceResource::client() {
 }
 
 void CWLSurfaceResource::enter(PHLMONITOR monitor) {
-    if (std::find(m_enteredOutputs.begin(), m_enteredOutputs.end(), monitor) != m_enteredOutputs.end())
+    if (std::ranges::find(m_enteredOutputs, monitor) != m_enteredOutputs.end())
         return;
 
     if UNLIKELY (!PROTO::outputs.contains(monitor->m_name)) {
@@ -292,10 +307,11 @@ void CWLSurfaceResource::enter(PHLMONITOR monitor) {
     m_enteredOutputs.emplace_back(monitor);
 
     m_resource->sendEnter(output->getResource().get());
+    m_events.enter.emit(monitor);
 }
 
 void CWLSurfaceResource::leave(PHLMONITOR monitor) {
-    if UNLIKELY (std::find(m_enteredOutputs.begin(), m_enteredOutputs.end(), monitor) == m_enteredOutputs.end())
+    if UNLIKELY (std::ranges::find(m_enteredOutputs, monitor) == m_enteredOutputs.end())
         return;
 
     auto output = PROTO::outputs.at(monitor->m_name)->outputResourceFrom(m_client);
@@ -308,6 +324,7 @@ void CWLSurfaceResource::leave(PHLMONITOR monitor) {
     std::erase(m_enteredOutputs, monitor);
 
     m_resource->sendLeave(output->getResource().get());
+    m_events.leave.emit(monitor);
 }
 
 void CWLSurfaceResource::sendPreferredTransform(wl_output_transform t) {
@@ -437,8 +454,8 @@ void CWLSurfaceResource::map() {
 
     frame(Time::steadyNow());
 
-    m_current.bufferDamage = CBox{{}, {INT32_MAX, INT32_MAX}};
-    m_pending.bufferDamage = CBox{{}, {INT32_MAX, INT32_MAX}};
+    m_current.bufferDamage = CBox{{}, m_current.bufferSize};
+    m_pending.bufferDamage = CBox{{}, m_pending.bufferSize};
 }
 
 void CWLSurfaceResource::unmap() {
@@ -523,6 +540,21 @@ void CWLSurfaceResource::commitState(SSurfaceState& state) {
         dropCurrentBuffer();
 }
 
+SImageDescription CWLSurfaceResource::getPreferredImageDescription() {
+    auto parent = m_self;
+    if (parent->m_role->role() == SURFACE_ROLE_SUBSURFACE) {
+        auto subsurface = ((CSubsurfaceRole*)parent->m_role.get())->m_subsurface.lock();
+        parent          = subsurface->t1Parent();
+    }
+    WP<CMonitor> monitor;
+    if (parent->m_enteredOutputs.size() == 1)
+        monitor = parent->m_enteredOutputs[0];
+    else if (m_hlSurface.valid() && m_hlSurface->getWindow())
+        monitor = m_hlSurface->getWindow()->m_monitor;
+
+    return monitor ? monitor->m_imageDescription : g_pCompositor->getPreferredImageDescription();
+}
+
 void CWLSurfaceResource::updateCursorShm(CRegion damage) {
     if (damage.empty())
         return;
@@ -563,13 +595,13 @@ void CWLSurfaceResource::updateCursorShm(CRegion damage) {
 
 void CWLSurfaceResource::presentFeedback(const Time::steady_tp& when, PHLMONITOR pMonitor, bool discarded) {
     frame(when);
-    auto FEEDBACK = makeShared<CQueuedPresentationData>(m_self.lock());
+    auto FEEDBACK = makeUnique<CQueuedPresentationData>(m_self.lock());
     FEEDBACK->attachMonitor(pMonitor);
     if (discarded)
         FEEDBACK->discarded();
     else
         FEEDBACK->presented();
-    PROTO::presentation->queueData(FEEDBACK);
+    PROTO::presentation->queueData(std::move(FEEDBACK));
 }
 
 CWLCompositorResource::CWLCompositorResource(SP<CWlCompositor> resource_) : m_resource(resource_) {
