@@ -21,29 +21,26 @@ CWLSubsurfaceResource::CWLSubsurfaceResource(SP<CWlSubsurface> resource_, SP<CWL
         if (!m_parent)
             return;
 
-        auto pushAboveIndex = [this](int idx) -> void {
-            for (auto const& c : m_parent->m_subsurfaces) {
-                if (c->m_zIndex >= idx)
-                    c->m_zIndex++;
-            }
-        };
-
         std::erase_if(m_parent->m_subsurfaces, [this](const auto& e) { return e == m_self || !e; });
 
-        auto it = std::find(m_parent->m_subsurfaces.begin(), m_parent->m_subsurfaces.end(), SURF);
+        std::ranges::for_each(m_parent->m_subsurfaces, [](const auto& e) { e->m_zIndex *= 2; });
+
+        auto it = std::ranges::find_if(m_parent->m_subsurfaces, [SURF](const auto& s) { return s->m_surface == SURF; });
 
         if (it == m_parent->m_subsurfaces.end()) {
             LOGM(ERR, "Invalid surface reference in placeAbove, likely parent");
-            pushAboveIndex(1);
             m_parent->m_subsurfaces.emplace_back(m_self);
             m_zIndex = 1;
         } else {
-            pushAboveIndex((*it)->m_zIndex);
-            m_zIndex = (*it)->m_zIndex;
+            m_zIndex = (*it)->m_zIndex + 1;
             m_parent->m_subsurfaces.emplace_back(m_self);
         }
 
-        std::sort(m_parent->m_subsurfaces.begin(), m_parent->m_subsurfaces.end(), [](const auto& a, const auto& b) { return a->m_zIndex < b->m_zIndex; });
+        std::ranges::sort(m_parent->m_subsurfaces, [](const auto& a, const auto& b) { return a->m_zIndex < b->m_zIndex; });
+
+        for (size_t i = 0; i < m_parent->m_subsurfaces.size(); ++i) {
+            m_parent->m_subsurfaces.at(i)->m_zIndex = i;
+        }
     });
 
     m_resource->setPlaceBelow([this](CWlSubsurface* r, wl_resource* surf) {
@@ -52,32 +49,29 @@ CWLSubsurfaceResource::CWLSubsurfaceResource(SP<CWlSubsurface> resource_, SP<CWL
         if (!m_parent)
             return;
 
-        auto pushBelowIndex = [this](int idx) -> void {
-            for (auto const& c : m_parent->m_subsurfaces) {
-                if (c->m_zIndex <= idx)
-                    c->m_zIndex--;
-            }
-        };
-
         std::erase_if(m_parent->m_subsurfaces, [this](const auto& e) { return e == m_self || !e; });
 
-        auto it = std::find(m_parent->m_subsurfaces.begin(), m_parent->m_subsurfaces.end(), SURF);
+        std::ranges::for_each(m_parent->m_subsurfaces, [](const auto& e) { e->m_zIndex *= 2; });
+
+        auto it = std::ranges::find_if(m_parent->m_subsurfaces, [SURF](const auto& s) { return s->m_surface == SURF; });
 
         if (it == m_parent->m_subsurfaces.end()) {
             LOGM(ERR, "Invalid surface reference in placeBelow, likely parent");
-            pushBelowIndex(-1);
             m_parent->m_subsurfaces.emplace_back(m_self);
             m_zIndex = -1;
         } else {
-            pushBelowIndex((*it)->m_zIndex);
-            m_zIndex = (*it)->m_zIndex;
+            m_zIndex = (*it)->m_zIndex - 1;
             m_parent->m_subsurfaces.emplace_back(m_self);
         }
 
-        std::sort(m_parent->m_subsurfaces.begin(), m_parent->m_subsurfaces.end(), [](const auto& a, const auto& b) { return a->m_zIndex < b->m_zIndex; });
+        std::ranges::sort(m_parent->m_subsurfaces, [](const auto& a, const auto& b) { return a->m_zIndex < b->m_zIndex; });
+
+        for (size_t i = 0; i < m_parent->m_subsurfaces.size(); ++i) {
+            m_parent->m_subsurfaces.at(i)->m_zIndex = i;
+        }
     });
 
-    m_listeners.commitSurface = m_surface->m_events.commit.registerListener([this](std::any d) {
+    m_listeners.commitSurface = m_surface->m_events.commit.listen([this] {
         if (m_surface->m_current.texture && !m_surface->m_mapped) {
             m_surface->map();
             m_surface->m_events.map.emit();
@@ -116,8 +110,7 @@ Vector2D CWLSubsurfaceResource::posRelativeToParent() {
     // surfaces we've visited and if we hit a surface we've visited we bail out.
     std::vector<SP<CWLSurfaceResource>> surfacesVisited;
 
-    while (surf->m_role->role() == SURFACE_ROLE_SUBSURFACE &&
-           std::find_if(surfacesVisited.begin(), surfacesVisited.end(), [surf](const auto& other) { return surf == other; }) == surfacesVisited.end()) {
+    while (surf->m_role->role() == SURFACE_ROLE_SUBSURFACE && std::ranges::find_if(surfacesVisited, [surf](const auto& other) { return surf == other; }) == surfacesVisited.end()) {
         surfacesVisited.emplace_back(surf);
         auto subsurface = ((CSubsurfaceRole*)m_parent->m_role.get())->m_subsurface.lock();
         pos += subsurface->m_position;
@@ -134,8 +127,7 @@ SP<CWLSurfaceResource> CWLSubsurfaceResource::t1Parent() {
     SP<CWLSurfaceResource>              surf = m_parent.lock();
     std::vector<SP<CWLSurfaceResource>> surfacesVisited;
 
-    while (surf->m_role->role() == SURFACE_ROLE_SUBSURFACE &&
-           std::find_if(surfacesVisited.begin(), surfacesVisited.end(), [surf](const auto& other) { return surf == other; }) == surfacesVisited.end()) {
+    while (surf->m_role->role() == SURFACE_ROLE_SUBSURFACE && std::ranges::find_if(surfacesVisited, [surf](const auto& other) { return surf == other; }) == surfacesVisited.end()) {
         surfacesVisited.emplace_back(surf);
         auto subsurface = ((CSubsurfaceRole*)m_parent->m_role.get())->m_subsurface.lock();
         surf            = subsurface->m_parent.lock();
