@@ -5,43 +5,43 @@
 #include "../../helpers/sync/SyncTimeline.hpp"
 #include <xf86drm.h>
 
-CWLBufferResource::CWLBufferResource(WP<CWlBuffer> resource_) : resource(resource_.lock()) {
+CWLBufferResource::CWLBufferResource(WP<CWlBuffer> resource_) : m_resource(resource_.lock()) {
     if UNLIKELY (!good())
         return;
 
-    resource->setOnDestroy([this](CWlBuffer* r) {
-        if (buffer.expired())
+    m_resource->setOnDestroy([this](CWlBuffer* r) {
+        if (m_buffer.expired())
             return;
-        buffer->events.destroy.emit();
+        m_buffer->events.destroy.emit();
     });
-    resource->setDestroy([this](CWlBuffer* r) {
-        if (buffer.expired())
+    m_resource->setDestroy([this](CWlBuffer* r) {
+        if (m_buffer.expired())
             return;
-        buffer->events.destroy.emit();
+        m_buffer->events.destroy.emit();
     });
 
-    resource->setData(this);
+    m_resource->setData(this);
 }
 
 bool CWLBufferResource::good() {
-    return resource->resource();
+    return m_resource->resource();
 }
 
 void CWLBufferResource::sendRelease() {
-    resource->sendRelease();
+    m_resource->sendRelease();
 }
 
 wl_resource* CWLBufferResource::getResource() {
-    return resource->resource();
+    return m_resource->resource();
 }
 
 SP<CWLBufferResource> CWLBufferResource::fromResource(wl_resource* res) {
     auto data = (CWLBufferResource*)(((CWlBuffer*)wl_resource_get_user_data(res))->data());
-    return data ? data->self.lock() : nullptr;
+    return data ? data->m_self.lock() : nullptr;
 }
 
 SP<CWLBufferResource> CWLBufferResource::create(WP<CWlBuffer> resource) {
-    auto p  = SP<CWLBufferResource>(new CWLBufferResource(resource));
-    p->self = p;
+    auto p    = SP<CWLBufferResource>(new CWLBufferResource(resource));
+    p->m_self = p;
     return p;
 }
