@@ -199,8 +199,9 @@ bool CXWaylandServer::tryOpenSockets() {
                 continue;
             }
 
-            const std::string pidStr = std::to_string(getpid());
-            if (write(fd.get(), pidStr.c_str(), pidStr.length()) != (long)pidStr.length()) {
+            const std::string pidStr = std::format("{:010d}\n", getpid());
+            ASSERT(pidStr.length() == 11);
+            if (write(fd.get(), pidStr.c_str(), 11) != 11L) {
                 safeRemove(lockPath);
                 continue;
             }
@@ -298,7 +299,7 @@ void CXWaylandServer::runXWayland(CFileDescriptor& notifyFD) {
         _exit(EXIT_FAILURE);
     }
 
-    auto cmd = std::format("Xwayland {} -rootless -core -listenfd {} -listenfd {} -displayfd {} -wm {}", m_displayName, m_xFDs[0].get(), m_xFDs[1].get(), notifyFD.get(),
+    auto cmd = std::format("exec Xwayland {} -rootless -core -listenfd {} -listenfd {} -displayfd {} -wm {}", m_displayName, m_xFDs[0].get(), m_xFDs[1].get(), notifyFD.get(),
                            m_xwmFDs[1].get());
 
     auto waylandSocket = std::format("{}", m_waylandFDs[1].get());
